@@ -1,5 +1,5 @@
 /* This file is part of GNU cflow
-   Copyright (C) 1997, 2005, 2007, 2009, 2010, 2011 Sergey Poznyakoff
+   Copyright (C) 1997-2021 Sergey Poznyakoff
  
    GNU cflow is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,10 +11,8 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public
-   License along with GNU cflow; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <cflow.h>
 #include <parser.h>
@@ -394,16 +392,26 @@ tree_output()
 	       separator();
 	  }
      } else {
-	  main_sym = lookup(start_name);
+	  void *state;
+	  
+	  main_sym = first_starter(&state);
 	  if (main_sym) {
-	       direct_tree(0, 0, main_sym);
-	       separator();
-	  } else {
+	       do {
+		   direct_tree(0, 0, main_sym);
+		   separator();
+	       } while ((main_sym = next_starter(&state)) != NULL);
+	  } else if (!all_functions) {
+	       all_functions = 1;
+	  }
+
+	  if (all_functions) {
 	       for (i = 0; i < num; i++) {
-		    if (symbols[i]->callee == NULL)
-			 continue;
-		    direct_tree(0, 0, symbols[i]);
-		    separator();
+		    if (symbols[i]->flag != symbol_start
+			&& symbols[i]->source
+			&& (all_functions > 1 || symbols[i]->caller == NULL)) {
+			 direct_tree(0, 0, symbols[i]);
+			 separator();
+		    }
 	       }
 	  }
      }
@@ -421,7 +429,7 @@ output()
      } else {
 	  outfile = fopen(outname, "w");
 	  if (!outfile)
-	       error(2, errno, _("cannot open file `%s'"), outname);
+	       error(EX_FATAL, errno, _("cannot open file `%s'"), outname);
      } 
      
      set_level_mark(0, 0);
@@ -429,6 +437,7 @@ output()
 	  xref_output();
      }
      if (print_option & PRINT_TREE) {
+	  eliminate_non_targets();
 	  tree_output();
      }
      fclose(outfile);
